@@ -18,6 +18,13 @@ from pg8000.converters import (
 )
 from pg8000.exceptions import DatabaseError, Error, InterfaceError
 
+try:
+    from importlib.metadata import version
+except ImportError:
+    from importlib_metadata import version
+
+ver = version("pg8000")
+
 
 def pack_funcs(fmt):
     struc = Struct(f"!{fmt}")
@@ -132,8 +139,8 @@ RESPONSE_LINE = "L"
 RESPONSE_ROUTINE = "R"
 
 IDLE = b"I"
-IDLE_IN_TRANSACTION = b"T"
-IDLE_IN_FAILED_TRANSACTION = b"E"
+IN_TRANSACTION = b"T"
+IN_FAILED_TRANSACTION = b"E"
 
 
 class CoreConnection:
@@ -336,7 +343,8 @@ class CoreConnection:
 
         try:
 
-            code = self.error = None
+            code = None
+            context = Context(None)
             while code not in (READY_FOR_QUERY, ERROR_RESPONSE):
 
                 try:
@@ -344,16 +352,16 @@ class CoreConnection:
                 except struct.error as e:
                     raise InterfaceError("network error") from e
 
-                self.message_types[code](self._read(data_len - 4), None)
+                self.message_types[code](self._read(data_len - 4), context)
 
-            if self.error is not None:
-                raise self.error
+            if context.error is not None:
+                raise context.error
 
         except Error as e:
             self.close()
             raise e
 
-        self.in_transaction = False
+        self._transaction_status = None
 
     def register_out_adapter(self, typ, out_func):
         self.py_types[typ] = out_func
@@ -368,10 +376,10 @@ class CoreConnection:
             if s != b""
         }
 
-        self.error = DatabaseError(msg)
+        context.error = DatabaseError(msg)
 
     def handle_EMPTY_QUERY_RESPONSE(self, data, context):
-        self.error = DatabaseError("query was empty")
+        pass
 
     def handle_CLOSE_COMPLETE(self, data, context):
         pass
@@ -589,7 +597,7 @@ class CoreConnection:
             )
 
     def handle_READY_FOR_QUERY(self, data, context):
-        self.in_transaction = data != IDLE
+        self._transaction_status = data
 
     def handle_BACKEND_KEY_DATA(self, data, context):
         self._backend_key_data = data
@@ -644,7 +652,7 @@ class CoreConnection:
         self._send_message(QUERY, sql.encode(self._client_encoding) + NULL_BYTE)
 
     def execute_simple(self, statement):
-        context = Context()
+        context = Context(statement)
 
         self.send_QUERY(statement)
         self._flush()
@@ -653,7 +661,7 @@ class CoreConnection:
         return context
 
     def execute_unnamed(self, statement, vals=(), oids=(), stream=None):
-        context = Context(stream=stream)
+        context = Context(statement, stream=stream)
 
         self.send_PARSE(NULL_BYTE, statement, oids)
         self._write(SYNC_MSG)
@@ -702,13 +710,15 @@ class CoreConnection:
             else:
                 raise e
 
-        context = Context()
+        context = Context(statement)
         self.handle_messages(context)
 
         return statement_name_bin, context.columns, context.input_funcs
 
-    def execute_named(self, statement_name_bin, params, columns, input_funcs):
-        context = Context(columns=columns, input_funcs=input_funcs)
+    def execute_named(
+        self, statement_name_bin, params, columns, input_funcs, statement
+    ):
+        context = Context(columns=columns, input_funcs=input_funcs, statement=statement)
 
         self.send_BIND(statement_name_bin, params)
         self.send_EXECUTE()
@@ -758,6 +768,11 @@ class CoreConnection:
         pass
 
     def handle_COMMAND_COMPLETE(self, data, context):
+        if self._transaction_status == IN_FAILED_TRANSACTION and context.error is None:
+            sql = context.statement.strip().rstrip(";").rstrip().upper()
+            if sql != "ROLLBACK":
+                context.error = InterfaceError("in failed transaction block")
+
         values = data[:-1].split(b" ")
         try:
             row_count = int(values[-1])
@@ -783,7 +798,7 @@ class CoreConnection:
         context.rows.append(row)
 
     def handle_messages(self, context):
-        code = self.error = None
+        code = None
 
         while code != READY_FOR_QUERY:
 
@@ -794,8 +809,8 @@ class CoreConnection:
 
             self.message_types[code](self._read(data_len - 4), context)
 
-        if self.error is not None:
-            raise self.error
+        if context.error is not None:
+            raise context.error
 
     def close_prepared_statement(self, statement_name_bin):
         """https://www.postgresql.org/docs/current/protocol-message-formats.html"""
@@ -803,7 +818,7 @@ class CoreConnection:
         self._write(FLUSH_MSG)
         self._write(SYNC_MSG)
         self._flush()
-        context = Context()
+        context = Context(None)
         self.handle_messages(context)
         self._statement_nums.remove(statement_name_bin)
 
@@ -831,9 +846,11 @@ class CoreConnection:
 
 
 class Context:
-    def __init__(self, stream=None, columns=None, input_funcs=None):
+    def __init__(self, statement, stream=None, columns=None, input_funcs=None):
+        self.statement = statement
         self.rows = None if columns is None else []
         self.row_count = -1
         self.columns = columns
         self.stream = stream
         self.input_funcs = [] if input_funcs is None else input_funcs
+        self.error = None
