@@ -3,7 +3,7 @@ import warnings
 
 import pytest
 
-from pg8000.dbapi import DatabaseError, InterfaceError, connect
+from pg8000.dbapi import DatabaseError, InterfaceError, __version__, connect
 
 
 def test_unix_socket_missing():
@@ -211,3 +211,94 @@ def test_closed_connection(db_kwargs):
         cursor.execute("VALUES ('hw1'::text)")
 
     warnings.resetwarnings()
+
+
+def test_version():
+    try:
+        from importlib.metadata import version
+    except ImportError:
+        from importlib_metadata import version
+
+    ver = version("pg8000")
+
+    assert __version__ == ver
+
+
+@pytest.mark.parametrize(
+    "commit",
+    [
+        "commit",
+        "COMMIT;",
+    ],
+)
+def test_failed_transaction_commit_sql(cursor, commit):
+    cursor.execute("create temporary table tt (f1 int primary key)")
+    cursor.execute("begin")
+    try:
+        cursor.execute("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    with pytest.raises(InterfaceError):
+        cursor.execute(commit)
+
+
+def test_failed_transaction_commit_method(con, cursor):
+    cursor.execute("create temporary table tt (f1 int primary key)")
+    cursor.execute("begin")
+    try:
+        cursor.execute("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    with pytest.raises(InterfaceError):
+        con.commit()
+
+
+@pytest.mark.parametrize(
+    "rollback",
+    [
+        "rollback",
+        "rollback;",
+        "ROLLBACK ;",
+    ],
+)
+def test_failed_transaction_rollback_sql(cursor, rollback):
+    cursor.execute("create temporary table tt (f1 int primary key)")
+    cursor.execute("begin")
+    try:
+        cursor.execute("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    cursor.execute(rollback)
+
+
+def test_failed_transaction_rollback_method(cursor, con):
+    cursor.execute("create temporary table tt (f1 int primary key)")
+    cursor.execute("begin")
+    try:
+        cursor.execute("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    con.rollback()
+
+
+@pytest.mark.parametrize(
+    "sql",
+    [
+        "BEGIN",
+        "select * from tt;",
+    ],
+)
+def test_failed_transaction_sql(cursor, sql):
+    cursor.execute("create temporary table tt (f1 int primary key)")
+    cursor.execute("begin")
+    try:
+        cursor.execute("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    with pytest.raises(DatabaseError):
+        cursor.execute(sql)
