@@ -4,7 +4,7 @@ from datetime import time as Time
 
 import pytest
 
-from pg8000.native import Connection, DatabaseError, InterfaceError
+from pg8000.native import Connection, DatabaseError, InterfaceError, __version__
 
 
 def test_unix_socket_missing():
@@ -218,3 +218,71 @@ def test_network_error_on_connect(db_kwargs, mocker):
     mocker.patch("pg8000.core.ci_unpack", side_effect=struct.error())
     with pytest.raises(InterfaceError, match="network error"):
         Connection(**db_kwargs)
+
+
+def test_version():
+    try:
+        from importlib.metadata import version
+    except ImportError:
+        from importlib_metadata import version
+
+    v = version("pg8000")
+
+    assert __version__ == v
+
+
+@pytest.mark.parametrize(
+    "commit",
+    [
+        "commit",
+        "COMMIT;",
+    ],
+)
+def test_failed_transaction_commit(con, commit):
+    con.run("create temporary table tt (f1 int primary key)")
+    con.run("begin")
+    try:
+        con.run("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    with pytest.raises(InterfaceError):
+        con.run(commit)
+
+
+@pytest.mark.parametrize(
+    "rollback",
+    [
+        "rollback",
+        "rollback;",
+        "ROLLBACK ;",
+    ],
+)
+def test_failed_transaction_rollback(con, rollback):
+    con.run("create temporary table tt (f1 int primary key)")
+    con.run("begin")
+    try:
+        con.run("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    con.run(rollback)
+
+
+@pytest.mark.parametrize(
+    "sql",
+    [
+        "BEGIN",
+        "select * from tt;",
+    ],
+)
+def test_failed_transaction_sql(con, sql):
+    con.run("create temporary table tt (f1 int primary key)")
+    con.run("begin")
+    try:
+        con.run("insert into tt(f1) values(null)")
+    except DatabaseError:
+        pass
+
+    with pytest.raises(DatabaseError):
+        con.run(sql)
